# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# clase core de las funciones nativas y alguna que otra función

import time
import random
import os


class ZynkNativeFunc: #emulador de funciones
    def __init__(self, func):
        self.func = func
    def call(self, interpreter, args):
        return self.func(args)
    def __repr__(self):
        return f"<native fn>"

#tiempo

def clock(args):
    return time.time()
def slep(args):
    time.sleep(args[0])
    return None

#random
def choic(args):
    return random.choice(args[0])
def randi(args):
    return random.randint(min(int(args[0]), int(args[1])), max(int(args[0]), int(args[1])))
def randf(args):
    return random.uniform(min(args[0], args[1]), max(args[0], args[1]))
def rand(args):
    return random.random()
#arrays

def lenght(args):
    return len(args[0])
def get_index(args):
    return args[0][int(args[1])]
def set_index(args):
    args[0][int(args[1])]=args[2]
    return None
def push(args):
    args[0].append(args[1])
    return None
def pup(args):
    args[0].pop(int(args[1]))
    return None

#FUNCIONES DE IO y otras cosas, hay que recordar que esto son modulos al fin de al cabo
#el interprete de C no tendra todas estas, tendra otras
#jijijiji, ya me pondre a hacer el bytecode
#

#
#  args[0] es el nombre
# cuando sea necesario args[1] es la información

def write_file(args):
    name = args[0]
    data = args[1]
    with open(name, "w") as f:
        f.write(data)
    return None
def read_file(args):
    name = args[0]
    with open(name, "r") as f:
        data = f.read()
    return data
def write_bytes(args):
    name = args[0]
    data = args[1]
    with open(name, "wb") as f:
        f.write(data)
    return None
def read_bytes(args):
    name = args[0]
    with open(name, "rb") as f:
        data = f.read()
    return data
def makedir(args):
    os.makedirs(args[0])
    return None
def listdir(args):
    return os.listdir()
def pwd(args):
    return os.getcwd()
def cwd(args):
    os.chdir(args[0])
    return None
def rmdir(args):
    os.removedirs(args[0])
    return None
def remove(args):
    os.remove(args[0])
    return None


# float a str y viceversa
def tfloat(args):
    return float(args[0])
def tstr(args):
    return str(args[0])
def tint(args):
    return int(args[0])

# input y print
def nprint(args):
    print(args[0])
    return None
def ninput(args):
    return input(args[0])

# time
nclock = ZynkNativeFunc(clock)
nsleep = ZynkNativeFunc(slep)

# arrays
nlenght = ZynkNativeFunc(lenght)
nget_index = ZynkNativeFunc(get_index)
nset_index = ZynkNativeFunc(set_index)
npush = ZynkNativeFunc(push)
npop = ZynkNativeFunc(pup)

#archivos
nwf = ZynkNativeFunc(write_file)
nrf = ZynkNativeFunc(read_file)
nwb = ZynkNativeFunc(write_bytes)
nrb = ZynkNativeFunc(read_bytes)

# conversion

ntf = ZynkNativeFunc(tfloat)
nts = ZynkNativeFunc(tstr)
nti = ZynkNativeFunc(tint)

#números aleatorios

nchoice = ZynkNativeFunc(choic)
nrandint = ZynkNativeFunc(randi)
nrandf = ZynkNativeFunc(randf)
nrandom = ZynkNativeFunc(rand)


#other filesystem interactions
nmakedir = ZynkNativeFunc(makedir)
nlistdir = ZynkNativeFunc(listdir)
npwd = ZynkNativeFunc(pwd)
ncwd = ZynkNativeFunc(cwd)
nrmdir = ZynkNativeFunc(rmdir)
nremove = ZynkNativeFunc(remove)

# input and print
ninp = ZynkNativeFunc(ninput)
npri = ZynkNativeFunc(nprint)

def add_natives(eval, funcs):
    for k, v in funcs.items():
        eval.env.define(k, v)


# CORE FUNCS
core_funcs = {
    "clock":nclock,
    "len":nlenght,
    "get_index":nget_index,
    "set_index":nset_index,
    "push":npush,
    "write":nwf,
    "read":nrf,
    "write_bytes":nwb,
    "read_bytes":nrb,
    "str":nts,
    "float":ntf,
    "int":nti,
    "pop":npop,
    "sleep":nsleep,
    "choice":nchoice,
    "randi":nrandint,
    "randf":nrandf,
    "rand":nrandom,
    "mkdir":nmakedir,
    "listdir":nlistdir,
    "pwd":npwd,
    "cwd":ncwd,
    "rmdir":nrmdir,
    "remove":nremove,
    "print":npri,
    "input":ninp
}


# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3
import sys

class ZynkError(Exception):
    def __init__(self, line, column, error):
        self.line =line
        self.column = column
        self.error = error
        self.msg = f"[line : {line}, column : {column}] : Error : {error}"
        super().__init__(self.msg)
    def print_error(self):
        print(self.msg, file=sys.stderr)
    def __str__(self):
        return self.msg
    def __repr__(self):
        return self.msg
    
class EvalError(ZynkError): # jajajja añadiendo un error nuevo, quien diria que hay que programar errores ¿no? xD
    def __init__(self, nexpr, error):
        self.nexpr = nexpr
        self.error = error
        self.column = None
        super().__init__(self.nexpr, self.column, self.error)

class ParserError(ZynkError): # otro tipo de error nuevo, luego añadire errores más especificos, de momento me es suficiente así, pensar que pare de hacer un lenguaje de programación para hacer otro, lo bueno es que ahora voy mucho más rápido
    def __init__(self, token, token_pos, error):
        self.token = token
        self.pos = token_pos
        self.error = error
        super().__init__(token, token_pos, error)


# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from ..ast import eval, expressions, zenv, parser
from .. import tokens
from .. import errors
from . import initialize # plantillas para inicialización de recursos, bueno, en realidad de eso se encarga templates
from . import stack
from . import mem
from . import funcs


class Cynk(eval.ZynkLEval): # C Transpiler for Zynk
    def __init__(self, std_path=None, enclosing=None, debug=False, extension=".zl"):
        super().__init__(std_path, enclosing, debug, extension)
        self.code = []
        self.context = ""
        self.temp_count = 0
        self.prefix="zynk_func"
        self.stack=stack.Stack() # usamos las por defecto
        self.tables = mem.MemoryManager()
        self.fmaker = funcs.FuncMk(self)
        self.program_header = initialize.InitProgram(self)
    def eval(self, expression):
        expression.accept(self)
    def seval(self, expr):
        expr.accept(self)
        self.code.append(self.context)
        return self.code
    def emit_ctx(self, context):
        self.code.append(context)
        self.context = ""
    def pop_ctx(self):
        try:
            return self.context
        finally:
            self.context=""
    def emit(self, line):
        self.context += line + "\n"
    def visit_literal(self, expr):
        if expr.value==None:
            self.emit(self.stack.spush("zynkNull()"))
        elif isinstance(expr.value, bool):
            self.emit(self.stack.spush(f"zynkBool({str(expr.value).lower()})"))
        elif isinstance(expr.value, str):
            self.emit(self.stack.spush(f'zynkCreateString(&sysarena, "{expr.value}")'))
        elif isinstance(expr.value, float):
            self.emit(self.stack.spush(f"zynkNumber({expr.value})"))
        elif isinstance(expr.value, list):
            self.emit(self.stack.spush(f"zynkCreateArray(&sysarena, {len(expr.value)})"))
    def visit_binary(self, expr):
        expr.right.accept(self)
        expr.left.accept(self)
        if expr.operand=="-":
            self.emit(self.stack.spush(f"zynkValuesSub({self.stack.spop()}, {self.stack.spop()})"))
        elif expr.operand=="+":
            self.emit(self.stack.spush(f"zynkValuesAdd({self.stack.spop()}, {self.stack.spop()})"))
        elif expr.operand=="*":
            self.emit(self.stack.spush(f"zynkValuesMul({self.stack.spop()}, {self.stack.spop()})"))
        elif expr.operand=="/":
            self.emit(self.stack.spush(f"zynkValuesDiv({self.stack.spop()}, {self.stack.spop()})"))
        elif expr.operand=="==":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesEqual({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="<":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesLess({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand==">":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesGreater({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="<=":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesLessEqual({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand==">=":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesGreaterEqual({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="and":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesAnd({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="or":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesOr({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="^":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesXor({self.stack.spop()}, {self.stack.spop()}))"))
        elif expr.operand=="!=":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesNotEqual({self.stack.spop()}, {self.stack.spop()}))"))
        else:
            error = errors.EvalError(expr, f"Invalid Binary Operand {expr.operand}")
            if self.debug:
                raise error
            error.print_error()
    def visit_unary(self, expr):
        operand = expr.operand
        right=expr.right.accept(self)

        if operand=="-":
            self.emit(self.stack.spush(f"zynkNumber(-{self.stack.spop()}.as.number)"))

        elif operand=="!":
            self.emit(self.stack.spush(f"zynkBool(zynkValuesNot({self.stack.spop()}))"))
        else:
            error = errors.EvalError(expr, f"Invalid Unary Operand {operand}!")
            if self.debug:
                raise error
            error.print_error()
    def visit_var_definition(self, expr):
        expr.expression.accept(self)
        self.emit(self.tables.emit_new(expr.name, self.stack))
    def visit_identifier(self, expr):
        self.emit(self.tables.pget(expr.name, self.stack))
    def visit_func_definition(self, expr):
        self.program_header.add(self.fmaker.emit_func(expr))
        ret = self.stack.spush(f"zynkCreateNativeFunction(&sysarena, {expr.name}, (ZynkFuncPtr){self.prefix}_{expr.name})") + "\n" + "\t"
        ret += self.tables.emit_new(expr.name, self.stack)
        self.emit(ret)
    def visit_call_function(self, expr):
        self.emit("{")
        self.emit(f"Value __tmp__=zynkCreateArray(&sysarena, {len(expr.args)});")
        for arg in expr.args:
            arg.accept(self)
            self.emit(f"zynkArrayPush(&sysarena, __tmp__, {self.stack.spop()});")
        self.emit(self.stack.spush(f'zynkCallFunction(&sysarena, env, "{expr.name.name}", __tmp__)'))
        if expr.to is not None:
            self.emit(self.tables.emit_set(expr.to, self.stack))
        self.emit("}")
    def visit_grouping(self, expr):
        return self.eval(expr)
    def visit_block(self, expr):
        self.emit(self.tables.emit_nenv())
        for stmt in expr.body:
            self.eval(stmt)
        self.emit(self.tables.emit_ret_env())
    def visit_if(self, expr):
        self.eval(expr.condition)
        self.emit(f"if ({self.stack.spop()}.as.boolean)" + " {\n")
        self.eval(expr.then)
        self.emit("}\n")
        if expr.else_branch is not None:
            self.emit("else {\n")
            self.eval(expr.else_branch)
            self.emit("}\n")
    def visit_while(self, expr, origin=True):
        if origin:
            self.emit(self.tables.emit_nenv())
        self.eval(expr.condition)
        self.emit(f"while ({self.stack.spop()}.as.boolean)" + "{\n")
        self.eval(expr.body)
        self.eval(expr.condition)
        self.emit("}\n")
        if origin:
            self.emit(self.tables.emit_ret_env())
    def visit_for(self, expr):
        self.eval(self.tables.emit_nenv())
        self.eval(expr.initialized)
        expr.body.append(expr.increment)
        self.visit_while(expr, origin=False)
        self.eval(self.tables.emit_ret_env())
    def visit_return(self, expr):
        self.eval(expr.expression)
        self.emit(f"return {self.stack.spop()};\n")
    def visit_array_expr(self, expr):
        self.emit("{")
        self.emit(f"Value __tmp__ = zynkCreateArray(&sysarena, {len(expr.items)});")
        for item in expr.items:
            self.eval(item)
            self.emit(f"zynkArrayPush(&sysarena, __tmp__, {self.stack.spop()});")
        self.emit(self.stack.spush("__tmp__"))
        self.emit("}\n")
    def visit_break(self, expr):
        self.emit("break;")


# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .c_base import templates, stack, mem, initialize, funcs 
from .c_base.translator import Cynk 
from .frontend.lexer import ZynkLLexer 
from .ast.parser import ZynkLParser    
from .ast import expressions as zexpr 
from . import tokens 
from . import errors

class Transpiler:
    def __init__(self, config=None, debug=False):
        self.debug=debug
        self.config = {
            "stack_max": templates.stack_defaults["stack_max"],
            "index_type": templates.stack_defaults["index_type"],
            "num_arenas": templates.sysarena_defaults["num_arenas"],
            "memory_size": templates.sysarena_defaults["memory_size"],
            "a_or_b": templates.sysarena_defaults["a_or_b"], # "ANSI" o "STANDALONE"
            "table_cap": templates.zenv_defaults["table_cap"],
            "output_filename": "output.c",
            "main_function_name": "main",
            "prefix": "zynk_func", 
        }
        if config:
            self.config.update(config)

        # Inicializa las herramientas del compilador
        self.lexer = ZynkLLexer(None, debug=True)
        self.parser = ZynkLParser(None, debug=True)
        
        self.cynk_translator = Cynk(
            debug=False
        )

        self.cynk_translator.prefix = self.config["prefix"] 
        self.main_program_c_code = ""
        self.defines = {} # Para el preprocesador simple

    def _simple_preprocess(self, source_code):
        lines = source_code.splitlines()
        processed_lines = []
        if_stack = [] 

        for line in lines:
            stripped_line = line.strip()

            if stripped_line.startswith("#define "):
                parts = stripped_line.split(maxsplit=2)
                if len(parts) >= 2:
                    key = parts[1]
                    value = parts[2] if len(parts) == 3 else ""
                    self.defines[key] = value
                continue 

            elif stripped_line.startswith("#ifdef "):
                macro = stripped_line.split(maxsplit=1)[1].strip()
                condition_met = macro in self.defines
                if_stack.append(condition_met)
                continue

            elif stripped_line.startswith("#ifndef "):
                macro = stripped_line.split(maxsplit=1)[1].strip()
                condition_met = macro not in self.defines
                if_stack.append(condition_met)
                continue

            elif stripped_line.startswith("#endif"):
                if if_stack:
                    if_stack.pop()
                continue
            
            if all(if_stack):
                processed_lines.append(line)
        
        return "\n".join(processed_lines)

    def transpile(self, src, includes=[]):
        zynk_source_code = ""
        for include in includes:
            with open(include, "r") as f:
                included = f.read()
            zynk_source_code += included + "\n"
        zynk_source_code += src
        processed_zynk_code = self._simple_preprocess(zynk_source_code)
        
        self.lexer.source = processed_zynk_code
        tokens_list = self.lexer.scan()
        if self.debug:
            for token in tokens_list:
                print(f"[ + {token} + ]")
        self.parser.tokens = tokens_list
        ast_root_statements = self.parser.parse()

        if self.debug:
            print(f"[ AST : {ast_root_statements} ]")
        self.cynk_translator.context = "" 
        ast_root_statements.accept(self.cynk_translator)
        self.main_program_c_code = self.cynk_translator.pop_ctx()

        final_c_output_parts = []

        final_c_output_parts.append(templates.stack_headers.format(
            stack_max=self.config["stack_max"],
            index_type=self.config["index_type"]
        ))
        final_c_output_parts.append(templates.sysarena_header.format(
            num_arenas=self.config["num_arenas"],
            memory_size=self.config["memory_size"],
            a_or_b=self.config["a_or_b"]
        ))
        final_c_output_parts.append(templates.zenv_headers.format(
            table_cap=self.config["table_cap"],
            cynk_header_names=templates.cynk_headers_defaults["cynk_header_names"]
        ))

        final_c_output_parts.append("\n// Zynk Function Declarations\n")
        final_c_output_parts.append(self.cynk_translator.program_header.emit())

        final_c_output_parts.append("\n// C Main Function and Zynk Entry Point\n")
        final_c_output_parts.append(f"""
int {self.config["main_function_name"]}() {{
    if (!cynkSysarenaInit()) {{
        return 1;
    }}
    
    ZynkEnv *env = cynkEnvCreate(NULL, CYNK_ENV_CAP, &sysarena); 
    if (env == NULL) {{
        return 1;
    }}
    
    // Código Zynk de nivel superior transpiled
    {self.main_program_c_code}

    cynkFreeEnv(env, &sysarena); 
    
    return 0;
}}
""")

        # Implementaciones de funciones Zynk
        final_c_output_parts.append("\n// Zynk Function Implementations\n")
        for func_c_code_block in self.cynk_translator.code:
             final_c_output_parts.append(func_c_code_block)
             final_c_output_parts.append("\n")

        return "\n".join(final_c_output_parts)

    def write_output(self, c_code_content):
        output_filepath = self.config["output_filename"]
        try:
            with open(output_filepath, "w") as f_out:
                f_out.write(c_code_content)
            print(f"C Code generated at '{output_filepath}'")
        except IOError as e:
            print(f"Error writing '{output_filepath}': {e}")


# --- Ejemplo de uso del Transpilador ---
if __name__ == "__main__":
    zynk_test_code = """
call print("Hola!");
"""

    my_transpiler = Transpiler(config={
        "output_filename": "final_program.c",
        "a_or_b": "ANSI", 
    }, debug=True)

    try:
        final_c_code = my_transpiler.transpile(zynk_test_code)
        my_transpiler.write_output(final_c_code)
    except Exception:
        print("[ Error ]")



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .. import errors
from . import expressions as zexpr
from . import zenv
from ..frontend import lexer
from . import parser
import os
from pathlib import Path
from ..native_funcs import core

class Visitor:
    def __init__(self):
        raise NotImplementedError()

class ZynkLEval(Visitor): # tengo que decir que amo el patron del visitante, es maravilloso
    def __init__(self, standard_lib_path=None, enclosing=None, debug=False, extension=".zl"):
        self.extension = extension
        self.debug = debug
        self.ret = None
        self.env = zenv.Enviroment(enclosing)
        if standard_lib_path is None:
            project_root = Path(__file__).resolve().parent.parent
            self.stdlib_path = project_root / "lib"
        else:
            self.stdlib_path = standard_lib_path
        core.add_natives(self, core.core_funcs)
        self._module_cache = {}
    def subenv(self):
        new = zenv.Enviroment(self.env)
        return new
    def new_subenv(self):
        self.switch(self.subenv())
    def switch(self, env):
        self.env = env
    def undo(self):
        self.env = self.env.enclosing
    def eval(self, expr):
        return expr.accept(self)
    def visit_literal(self, expr):
        return expr.value
    def visit_binary(self, expr):
        left = expr.left.accept(self)
        right = expr.right.accept(self)

        if expr.operand == "+":
            return left + right
        elif expr.operand == "*":
            return left * right
        elif expr.operand == "/":
            return left / right
        elif expr.operand == "-":
            return left - right
        elif expr.operand == "==":
            return left == right
        elif expr.operand == "<":
            return left < right
        elif expr.operand == ">":
            return left > right
        elif expr.operand == "<=":
            return left <= right
        elif expr.operand == ">=":
            return left >= right
        elif expr.operand == "and":
            return left and right
        elif expr.operand == "or":
            return left or right
        elif expr.operand == "^":
            return left ^ right
        elif expr.operand == "!=":
            return left != right
        else:
            error = errors.EvalError(expr, f"Invalid Binary Operand '{expr.operand}'!")
            if self.debug:
                raise error
            error.print_error()
    def visit_unary(self, expr):
        operand = expr.operand
        right = self.eval(expr.right)

        if operand == "!":
            return not right
        elif operand == "-":
            return - right
        else:
            error = errors.EvalError(expr, f"Invalid Unary Operand {expr.operand}!")
            if self.debug:
                raise error
            error.print_error()
    def visit_grouping(self, expr):
        return expr.expression.accept(self)
    def visit_var_definition(self, expr):
        self.env.define(expr.name, self.eval(expr.expression))
        return None
    def visit_identifier(self, expr):
        return self.env.get(expr.name)
    def visit_func_definition(self, expr):
        function = zexpr.ZynkFunc(expr)
        self.env.define(expr.name, function)
        return None
    def visit_call_function(self, expr):
        fname = expr.name.name
        if isinstance(expr.name, zexpr.MIdentifier):
            self.switch(self.env.get(expr.name.module).env)
        function = self.env.get(fname)
        if not hasattr(function, "call"):
            raise RuntimeError(f"Error : '{fname} is not a function!")
        args = []
        for arg in expr.args:
            args.append(self.eval(arg))
        ret = function.call(self, args)
        if isinstance(expr.name, zexpr.MIdentifier):
            self.undo()
        self.env.assign(expr.to, ret)
        return None
    def visit_block(self, expr):
        self.new_subenv()
        try:
            result = None
            for stmt in expr.body:
                result = self.eval(stmt)
            return result
        finally:
            self.undo()
    def visit_if(self, expr):
        if self.eval(expr.condition):
            return self.eval(expr.then)
        elif expr.else_branch is not None:
            return self.eval(expr.else_branch)
        return None
    def visit_while(self, expr):
        while self.eval(expr.condition):
            self.eval(expr.body)
        return None
    def visit_print(self, expr):
        value = self.eval(expr.expression)
        print(value)
        return None
    def visit_input(self, expr):
        prompt = ""
        if expr.expression is not None:
            prompt = str(self.eval(expr.expression))
        t = input(prompt)
        self.env.assign(expr.to, t)
        return t
    def visit_import(self, expr): # esto a sido una tortura
        cwdir = os.getcwd()
        filename = self.eval(expr.name) + self.extension
        if self.debug:
            print(f"[+] Trying to Load {self.eval(expr.name)} from current work dir [+]")
        if filename in os.listdir():
            filepath = os.path.join(cwdir, filename)
            if os.path.isfile(filepath):
                if self.debug:
                    print(f"[+] Loading module {self.eval(expr.name)} from {filepath} [+]")
            else:
                raise RuntimeError(f"Error : {filepath} is not a file")
        else:
            if self.debug:
                print("[+] Trying to Load from Standard Library [+]")
            filepath = os.path.join(self.stdlib_path, filename)
        if filepath in self._module_cache:
            module = self._module_cache[filepath]
            self.env.define(self.eval(expr.alias), module)
            if self.debug:
                print("[+] Module Loaded From Cache [+]")
            return None
        with open(filepath, "r") as f:
            if self.debug:
                print(f"[+] Loading {self.eval(expr.name)} from {filepath} [+]")
            lex = lexer.ZynkLLexer(f.read(), self.debug)
            tokens = lex.scan()
            if self.debug:
                print(f"[+] Analysis of {expr.name} module completed [+]")
            pars = parser.ZynkLParser(tokens, self.debug)
            parsed = pars.parse()
            if self.debug:
                print(f"[+] Module Parsed [+]")
                print(f"[+] Evaluating [+]")
            self.new_subenv()
            for expression in parsed:
                self.eval(expression)
            module = zexpr.Module(self.eval(expr.name), self.env)
            self.undo()
            self.env.define(self.eval(expr.alias), module)
            self._module_cache[filepath] = module
            return None

    def visit_for(self, expr):
        self.new_subenv()
        self.eval(expr.initialized)
        while self.eval(expr.condition):
            self.eval(expr.body)
            self.eval(expr.increment)
        return None
    def visit_var_assign(self, expr):
        self.env.assign(expr.name, self.eval(expr.expression))
        return None
    def visit_return(self, expr):
        self.ret = self.eval(expr.expression)
        return self.ret
    def visit_array_expr(self, expr):
        return [self.eval(element) for element in expr.items]
    def visit_index_expr(self, expr):
        array = self.eval(expr.array)
        index = self.eval(expr.index)
        print(array)
    
    # Validaciones
        if not isinstance(array, list):
            raise errors.EvalError(expr, f"Expected array, got {type(array)}")
        try:
            index = int(index)
        except (TypeError, ValueError):
            raise errors.EvalError(expr, f"Array index must be integer, got {type(index)}")
    
        try:
            return array[index]
        except IndexError:
            raise errors.EvalError(expr, f"Index {index} out of bounds")

    def visit_index_assign_expr(self, expr):
        array = self.eval(expr.array)
        index = self.eval(expr.index)
        value = self.eval(expr.value)
        print(array)
    
        if not isinstance(array, list):
            raise errors.EvalError(expr, f"Expected array, got {type(array)}")
        try:
            index = int(index)
        except (TypeError, ValueError):
            raise errors.EvalError(expr, f"Array index must be integer, got {type(index)}")
    
        try:
            array[index] = value
            return value
        except IndexError:
            raise errors.EvalError(expr, f"Index {index} out of bounds")
        
    # wow, añadi muchos metodos, me falta algo para arrays, aunque eso tambien en el lexer y expresiones deberia montar bucles for pues todavia no existen tambien algo para imports
    # esto es sencillo, creo que más tarde copiare esto pero en vez de interpretar que compile, estaria bien, de paso uso patrones de diseño
    # el funcionamiento de ZynkLite va a hacer que me replantee mejoras en ZynkPy


# sodio, yo montando un lenguaje de programación desde 0 para que tengais video xD, viva la república y viva la Asexualidad! :)
# realmente no creo que sea sano programar de 23:54 hasta 05:00 creo que deberia dormir más, pero bueno, programar es programar!



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import *
from .. import tokens, errors
from ..ast import *

class InitProgram:
    def __init__(self, trans):
        self.trans=trans
        self.headers= """
        // Headers Generated with InitProgram class from zynk_lite/c_base/initialize.py

        """

    def add(self, line):
        self.headers+=line+"\n"
    def add_func(self, fname): # para añadir las declaraciones, xD
        self.add(f"Value {self.trans.prefix}_{fname}(ArenaManager *manager, ZynkEnv *env, ZynkArray args);")
    def emit(self):
        return self.headers



from .ast import parser, expressions, eval, zenv
from .frontend import lexer
from .c_base import translator, initialize
from . import tokens, errors

test_expr_src = [
        "2 + 3",
        "5*2",
        "7-1",
        "5*3-1",
        "4+2*2-1+3",
        "4*2/3-1",
        "5*2-1/3*2",
        "7/2+0.5-1",
        "7 == 7",
        "5 - 3 > 5",
        "2 - 4 < 3",
        "2 < 3 and 5 > 3",
        "2 - 1 > 2 or 5 - 1 < 10",
        ]

if __name__ == "__main__":
    print("[+] Testing Mathematical Expressions [+]")
    for test in test_expr_src:
        print(f"[+] Expr : {test} [+]")
        zlexer = lexer.ZynkLLexer(test, debug=True)
        print("[+] Lexing... [+]")
        tokens = zlexer.scan()
        zparser = parser.AlgebraicParser(tokens, debug=True)
        print("[+] Lexing Process Finished [+]")
        for token in tokens:
            print(f"[+] : {token} : [+]")
        print("[+] Parsing... [+]")
        test_ast = zparser.parse()
        print("[+] Parsing Process Finished [+]")
        print(f"[+] AST (Abstract Syntax Tree) : {test_ast} [+]")
        print("[+] Transpiling to C... [+]")
        ztrans = translator.Cynk()
        ztrans.eval(test_ast)
        print("[+] Code Transpiled! [+]")
        print("[+] Current Context [+]")
        print(ztrans.context)
        print("[+] Final C Code [+]")
        print("\n".join(ztrans.code))



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import *
import sys
import os

def cynk_help():
    print("[ Use : cynk -s [source files] -o [out file] -i [includes] ]")
    print("[ Example : cynk -s my_program.zl -o my_program.c ]")
    print("[ Options ]")
    print("[ -s : source file ]")
    print("[ -o : out file ]")
    print("[ -i : files included ]")
    print("[ -v : sets the debug flag of the transpiler ]")

def main():
    help_flag=False
    source = None
    includes = []
    out_file = "out.c"
    debug=False
    alone=False
    if "--standalone" in sys.argv:
        sys.argv.remove("--standalone")
        alone=True
    if len(sys.argv) < 2:
        print("[ Error, at least 1 arg is needed, type with --help or with -h ]")
    arg_index=0
    while arg_index < len(sys.argv):
        arg=sys.argv[arg_index]
        if arg=="--help" or arg=="-h":
            help_flag=True
            break
        elif arg=="-s":
            arg_index+=1
            if arg_index >= len(sys.argv):
                raise Exception("[ Expected arg after '-s' ]")
            arg=sys.argv[arg_index]
            source = arg[:]
        elif arg=="-o":
            arg_index+=1
            if arg_index >= len(sys.argv):
                raise Exception("[ Expected arg after '-o' ]")
            arg=sys.argv[arg_index]
            out_file = arg[:]
        elif arg=="-i":
            while arg[0]!="-" and arg_index < len(sys.argv):
                arg=sys.argv[arg_index]
                includes.append(arg)
                arg_index+=1
            if arg[0]=="-":
                continue
        elif arg=="-v":
            debug=True
        arg_index+=1
    if help_flag:
        cynk_help()
    elif source is not None:
        if alone:
            config = {"a_or_b":"STANDALONE"}
        else:
            config = {"a_or_b":"ANSI"}
        config["output_filename"] = out_file
        trans = transpiler.Transpiler(config=config, debug=debug)
        with open(source, "r") as f:
            source_code = f.read()
        trans.transpile(source_code, includes)
    else:
        cynk_help()




# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import interpreter
from . import compiler
from . import errors
from . import tokens
from . import ast
from . import bytecode
from . import frontend
from . import native_funcs
from . import transpiler
from . import c_base
from . import test_trans



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .frontend import lexer
from .ast import eval
from .ast import parser

class ZynkLInterpreter:
    def __init__(self, standard_lib_path=None, enclosing=None, debug=False, extension=".zl"):
        self.evaluator = eval.ZynkLEval(standard_lib_path, enclosing, debug, extension)
        self.debug = debug
    def eval(self, code):
        lex = lexer.ZynkLLexer(code, self.debug)
        tokens = lex.scan()
        par = parser.ZynkLParser(tokens, self.debug)
        parsed = par.parse()
        self.evaluator.eval(parsed)
        if self.debug:
            print(tokens)
            print(parsed)
    def eval_file(self, filepath):
        with open(filepath, "r") as f:
            code = f.read()
        self.eval(code)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from src.zynk_lite import interpreter as intp
interpreter = intp.ZynkLInterpreter(debug=True)
case = """
func isAdult(age) {
	if (age >= 18) {
		return true;
	} else {
		return false;
	}
}

var r;
call isAdult(16) to r;
print r;

var h = [2, 3];
print h;
call push (h, 3);
print h;

call write("ns.txt", "hola");

"""
interpreter.eval(case)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .. import tokens
from .. import errors
from . import stack
from . import mem
from . import translator
from . import initialize

class FuncMk:
    def __init__(self, evaler):
        self.evaler=evaler
        self.tables=evaler.tables
        self.stack=evaler.stack
    def func_init(self, name):
        return f"Value {self.evaler.prefix}_{name}(ArenaManager *manager, ZynkEnv *env, ZynkArray *args)" + " {" + f"\n \t {self.tables.emit_nenv()}\n\tValue ret;\n\t"
    def emit_func(self, expr):
        self.evaler.program_header.add_func(expr.name)
        func_code = self.func_init(expr.name)
        func_code += self.unpack(expr.params)
        func_code += self.evaler.seval(expr.body)[-1]
        self.evaler.code.pop(-1)
        func_code += self.end_func()
        return func_code
    def end_func(self):
        return f"{self.tables.emit_ret_env()}\n" + "return ret;" + "\n}"
    def unpack_first(self):
        return "for (uint8_t i=0;i<args->len || i<256;i++) {\n\t\t" + self.stack.spush("args->value[i]") + "\n\t}\n\t"
    def unpack_second(self, names):
        ret = ""
        for name in reversed(names):
            ret += self.tables.emit_new(name, self.stack)
            ret += "\n\t"
        return ret
    def unpack(self, names):
        return self.unpack_first() + self.unpack_second(names)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .. import tokens
from . import expressions as zexpr
from .. import errors

class AlgebraicParser: # tremend descenso recursivo
    def __init__(self, tokens, debug=False):
        self.tokens = tokens
        self.debug = debug
        self.current = 0
        self.stop = False
    def parse(self):
        return self.parse_logic()
    def is_at_end(self):
        return self.current >= len(self.tokens)
    def advance(self):
        if not self.is_at_end():
            self.current += 1
        return self.tokens[self.current-1]
    def peek(self):
        if self.current+1 < len(self.tokens):
            return self.tokens[self.current+1]
        return None
    def eat(self, tipo):
        if not self.is_at_end() and self.tokens[self.current].type == tipo:
            self.advance()
            return True
        return False
    def prev(self):
        return self.tokens[self.current-1]
    def eat_more(self, *types):
        if not self.is_at_end() and self.tokens[self.current].type in types:
            self.advance()
            return True
        return False
    def parse_logic(self):
        fnode = self.parse_comp()
        while self.eat_more(tokens.TokenType.AND, tokens.TokenType.OR, tokens.TokenType.XOR):
            if self.current+1 < len(self.tokens) and self.peek().type==tokens.TokenType.SEMICOLON:
                break
            op = self.prev().lexem
            snode = self.parse_comp()
            fnode = zexpr.Binary(fnode, op, snode)
        return fnode
    def parse_comp(self):
        fnode = self.parse_expr()
        while self.eat_more(tokens.TokenType.EQUAL_EQUAL, tokens.TokenType.BANG_EQUAL, tokens.TokenType.LESS, tokens.TokenType.GREATER, tokens.TokenType.LESS_EQUAL, tokens.TokenType.GREATER_EQUAL):
            if self.current+1 < len(self.tokens) and self.peek().type==tokens.TokenType.SEMICOLON:
                break
            op = self.prev().lexem
            snode = self.parse_expr()
            fnode = zexpr.Binary(fnode, op, snode)
        return fnode
    def parse_expr(self):
        fnode = self.parse_term()
        while self.eat_more(tokens.TokenType.PLUS, tokens.TokenType.MINUS):
            if self.current+1 < len(self.tokens) and self.peek().type==tokens.TokenType.SEMICOLON:
                break
            op = self.prev().lexem
            snode = self.parse_term()
            fnode = zexpr.Binary(fnode, op, snode)
        return fnode
    def parse_term(self):
        fnode = self.parse_factor()
        while self.eat_more(tokens.TokenType.STAR, tokens.TokenType.SLASH):
            if self.current+1 < len(self.tokens) and self.peek().type==tokens.TokenType.SEMICOLON:
                break
            op=self.prev().lexem
            snode = self.parse_factor()
            fnode = zexpr.Binary(fnode, op, snode)
        return fnode

    def parse_factor(self):
        if self.eat_more(tokens.TokenType.MINUS, tokens.TokenType.BANG):
            op = self.prev().lexem
            return zexpr.Unary(op, self.parse_factor())
        elif self.eat(tokens.TokenType.LPAREN): # ya me olvidaba de esto
            expr = self.parse_logic()
            if not self.eat(tokens.TokenType.RPAREN):
                raise SyntaxError("Expected closing parenthesis")
            return expr
        elif self.eat_more(tokens.TokenType.STRING, tokens.TokenType.FLOAT, tokens.TokenType.BOOL, tokens.TokenType.NULL):
            return zexpr.Literal(self.prev().value)
        elif self.eat(tokens.TokenType.CALL):
            if not self.eat(tokens.TokenType.IDENTIFIER):
                raise SyntaxError("Expected Identifier")
            idtf = zexpr.Identifier(self.prev().lexem)
            if not self.eat(tokens.TokenType.LPAREN):
                raise SyntaxError("Expected Paren")
            args = self.get_args()
            args = [self.algebraic(arg) for arg in args if arg]
            if not self.eat(tokens.TokenType.SEMICOLON):
                raise SyntaxError("Expected Semicolon")
            return zexpr.CallFunc(idtf, args, None)
        elif self.eat(tokens.TokenType.IDENTIFIER):
            if self.eat(tokens.TokenType.DOT):
                if self.eat(tokens.TokenType.IDENTIFIER):
                    return zexpr.MIdentifier(self.tokens[self.current-3].lexem, self.prev().lexem)
                else:
                    raise SyntaxError("Expected identifier after a dot")
            return zexpr.Identifier(self.prev().lexem)
        elif self.eat(tokens.TokenType.LBRACKET):
            return self.parse_array()
        else:
            raise SyntaxError(f"Unexpected Token {self.tokens[self.current]}")
    def get_args(self):
        args = []
        arg = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.RPAREN):
                args.append(arg)
                break
            elif self.eat(tokens.TokenType.COMMA):
                args.append(arg)
                arg = []
            arg.append(self.advance())
        return args
    def parse_array(self):
        elements = []
        current_element = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.RBRACKET):
                elements.append(self.algebraic(current_element))
                break
            elif self.eat(tokens.TokenType.COMMA):
                elements.append(self.algebraic(current_element))
                current_element = []
            else:
                current_element.append(self.advance())
        return zexpr.ArrayExpr(elements)
    def algebraic(self, toks):
        psd = AlgebraicParser(toks, self.debug)
        return psd.parse()

# monte eso a escondidas a 4 am
# código clandestino


class ZynkLParser:
    def __init__(self, tokens, debug=False):
        self.debug = debug
        self.tokens = tokens
        self.current = 0
    def is_at_end(self):
        return self.current >= len(self.tokens)
    def advance(self):
        if not self.is_at_end():
            self.current += 1
        return self.tokens[self.current-1]
    def peek(self):
        if self.current+1 <= len(self.tokens):
            return self.tokens[self.current+1]
        return None
    def eat(self, tipe):
        if not self.is_at_end() and self.actual().type == tipe:
            self.advance()
            return True
        return False
    def prev(self):
        return self.tokens[self.current-1]
    def eat_more(self, *types):
        if not self.is_at_end() and self.actual().type in types:
            self.advance()
            return True
        return False
    def actual(self):
        if not self.is_at_end():
            return self.tokens[self.current]
        return None
    def parse_import(self):
        if self.eat_more(tokens.TokenType.IDENTIFIER, tokens.TokenType.STRING):
            first = self.algebraic([self.prev()])
            if self.eat(tokens.TokenType.AS):
                if self.eat_more(tokens.TokenType.IDENTIFIER, tokens.TokenType.STRING):
                    return zexpr.ImportExpr(first, self.algebraic([self.prev()]))
                else:
                    raise SyntaxError("Expected Identifier/String after as")
            else:
                return zexpr.ImportExpr(first, first) 
        else:
            raise SyntaxError("Expected Identifier/String After Import")
    def parse_identifier(self):
        first = self.prev()
        if self.eat(tokens.TokenType.DOT):
            if self.eat(tokens.TokenType.IDENTIFIER):
                p = self.prev().lexem
                return zexpr.MIdentifier(first.lexem, p)
            else:
                raise SyntaxError("Expected Identifier After dot")
        else:
            p = first.lexem
            return zexpr.Identifier(p)
    def algebraic(self, toks):
        psd = AlgebraicParser(toks, self.debug)
        return psd.parse()
    def parse(self):
        statements = []
        while not self.is_at_end():
            stmt = self.parse_stmt()
            if stmt is not None:
                statements.append(stmt)
        return zexpr.BlockExpr(statements)
    def parse_stmt(self):
        tok = self.actual()
        self.advance()
        if tok is None:
            return None
        elif tok.type==tokens.TokenType.BREAK:
            return zexpr.BreakExpr()
        elif tok.type==tokens.TokenType.RETURN:
            expre = self.parse_expression()
            return zexpr.ReturnExpr(expre)
        elif tok.type == tokens.TokenType.IMPORT:
            return self.parse_import()
        elif tok.type == tokens.TokenType.IDENTIFIER:
            first = self.parse_identifier()
            if self.eat(tokens.TokenType.EQUAL):
                expression = self.parse_expression()
                return zexpr.VarAssign(first.name, expression)
            else:
                raise SyntaxError("Unexpected Token after Identifier")
        elif tok.type == tokens.TokenType.IF:
           return self.parse_if()
        elif tok.type == tokens.TokenType.CALL:
            return self.parse_call()

        elif tok.type == tokens.TokenType.WHILE:
            return self.parse_while()

        elif tok.type == tokens.TokenType.FOR:
            return self.parse_for()

        elif tok.type == tokens.TokenType.FUNC:
            return self.parse_func()
        elif tok.type == tokens.TokenType.VAR:
            if self.eat(tokens.TokenType.IDENTIFIER):
                name = self.prev().lexem
                if self.eat(tokens.TokenType.EQUAL):
                    value = self.parse_expression()
                    return zexpr.VarDef(name, value)
                elif self.eat(tokens.TokenType.SEMICOLON):
                    return zexpr.VarDef(name, zexpr.Literal(None))
                else:
                    raise SyntaxError(f"Unexpected token {self.actual()}")
            else:
                raise SyntaxError("Expected identifier after 'var'")
        elif tok.type == tokens.TokenType.LBRACE:
            self.advance()
            return self.parse_block()
        elif tok.type == tokens.TokenType.SEMICOLON:
            self.advance()
            return None
        elif tok.type == tokens.TokenType.EOF:
            return None
    def parse_call(self):
        if not self.eat(tokens.TokenType.IDENTIFIER):
            raise SyntaxError("Expected Identifier")
        idtf = self.parse_identifier()
        if not self.eat(tokens.TokenType.LPAREN):
            raise SyntaxError("Expected Paren")
        args = self.get_args()
        args = [self.algebraic(arg) for arg in args if arg]
        if self.eat(tokens.TokenType.TO):
            to = self.advance().lexem
        else:
            to = None
        if not self.eat(tokens.TokenType.SEMICOLON):
            raise SyntaxError("Expected Semicolon")
        return zexpr.CallFunc(idtf, args, to)
    def until_to(self):
        until = []
        while not self.is_at_end():
            if self.eat_more(tokens.TokenType.TO, tokens.TokenType.SEMICOLON):
                break
            if self.eat(tokens.TokenType.EOF):
                raise SyntaxError("Unexpected EOF")
            until.append(self.advance())
        return until
    def parse_expression(self):
        expr = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.SEMICOLON):
                break
            elif self.eat(tokens.TokenType.EOF):
                raise SyntaxError("Unexpected EOF")
            else:
                expr.append(self.advance())
        return self.algebraic(expr)
    def parse_block(self):
        block = []
        braces = 1
        while not self.is_at_end():
            if self.eat(tokens.TokenType.RBRACE):
                braces -= 1
                if braces == 0:
                    break
                else:
                    block.append(self.prev())
            elif self.eat(tokens.TokenType.LBRACE):
                braces += 1
                block.append(self.prev())
            elif self.eat(tokens.TokenType.EOF):
                raise SyntaxError("Unexpected EOF")
            else:
                block.append(self.advance())
        subparse = ZynkLParser(block, self.debug)
        return zexpr.BlockExpr(subparse.parse())
    def parse_if(self):
        if not self.eat(tokens.TokenType.LPAREN):
            raise SyntaxError("Expected Paren after if")
        condition = self.parsinp()
        if not self.eat(tokens.TokenType.LBRACE):
            raise SyntaxError("Expected Brace after condition")
        then_branch = self.parse_block()
        else_branch = None

        if self.eat(tokens.TokenType.ELSE):
            if self.eat(tokens.TokenType.LBRACE):
                else_branch = self.parse_block()
            elif self.eat(tokens.TokenType.IF):
                # Manejar else if
                else_branch = self.parse_if()
            else:
                raise SyntaxError("Expected '{' or 'if' after 'else'")
    
        return zexpr.IfExpr(condition, then_branch, else_branch)
    def parse_while(self):
        if not self.eat(tokens.TokenType.LPAREN):
            raise SyntaxError("Expected Paren after while")
        condition = self.parsinp()
        if not self.eat(tokens.TokenType.LBRACE):
            raise SyntaxError("Expected Brace after condition")
        body = self.parse_block()
        return zexpr.WhileExpr(condition, body)
    def parse_for(self):
        if not self.eat(tokens.TokenType.LPAREN):
            raise SyntaxError("Expected Paren after for")
        args = self.get_args()
        name = args[0][0].lexem
        if args[0][1].lexem != "=":
            raise SyntaxError("Expected Equal")
        args[0] = zexpr.VarDef(name, self.algebraic(args[0][2:]))
        args[1] = self.algebraic(args[1])
        args[2] = zexpr.VarAssign(name, self.algebraic(args[2]))
        if not self.eat(tokens.TokenType.LBRACE):
            raise SyntaxError("Expected Brace after condition")
        body = self.parse_block()
        return zexpr.ForExpr(args[0], args[2], args[1], body)
    def parse_func(self):
        funcname = self.actual().lexem
        self.advance()
        if not self.eat(tokens.TokenType.LPAREN):
            raise SyntaxError("Expected Paren after Function Name")
        args = self.get_args()
        args = [arg[0].lexem for arg in args if len(arg) > 0]
        if not self.eat(tokens.TokenType.LBRACE):
            raise SyntaxError("Expected Brace after Params")
        body = self.parse_block()
        return zexpr.FuncDef(funcname, args, body)
    def parsinp(self):
        expr = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.RPAREN):
                break
            elif self.eat(tokens.TokenType.EOF):
                raise SyntaxError("Unexpected EOF")
            else:
                expr.append(self.advance())
        return self.algebraic(expr)
    def get_args(self):
        args = []
        arg = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.RPAREN):
                args.append(arg)
                break
            elif self.eat(tokens.TokenType.COMMA):
                args.append(arg)
                arg = []
            arg.append(self.advance())
        return args

        # el gran deepseek me ayudo un poco con alguna optimización y ahora con arrays, quiero pasar ya al bytecode
    def parse_array(self):
        elements = []
        arg = []
        while not self.is_at_end():
            if self.eat(tokens.TokenType.COMMA):
                elements.append(self.algebraic(arg))
                arg = []
            elif self.eat(tokens.TokenType.RBRACKET):
                arg.append(self.advance())
        return zexpr.ArrayExpr(elements)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from .. import errors
from .. import tokens

class ZynkLLexer:
    def __init__(self, source, debug=False, comillas='"'):
        self.debug = debug
        self.source = source # código fuente
        self.current = 0 # posición actual + 1
        self.start = 0 # donde empieza el token actual
        self.line = 1
        self.column = 1
        self.error = False
        self.tokens = [] # tokens de salida
        self.var_set = "abcdefghijklmnñopqrstuvwxyzABCDEFGHIJKLMNÑOPQRSTUVWXYZ_"
        self.comillas=comillas
        # fin de lo necesario actualmente
    def is_at_end(self): # para saber si llegamos al final del código
        return self.current >= len(self.source)
    def peek(self): # obtener siguiente caracter
        if self.is_at_end():
            return "\0"
        return self.source[self.current]
    def prev(self):
        return self.source[self.current-1] # caracter previo
    def advance(self): # avanzer
        self.current += 1
        self.column += 1
        return self.source[self.current-1]
    def match(self, expected): # consumir si es el caracter esperado
        if self.is_at_end() or self.peek() != expected:
            return False
        self.current += 1
        self.column += 1
        return True
    def match_sequence(self, seq): # Identificar Patrones
        ret_point = self.current
        first = self.prev()
        if first != seq[0]:
            return False
        i = 1
        while i < len(seq) and not self.is_at_end():
            if not self.match(seq[i]):
                self.current = ret_point
                return False
            i += 1
        if self.peek() not in self.var_set:
            return True
        self.current = ret_point
        return False
    def add_token(self, tipo, lexem="", value=None):
        self.tokens.append(tokens.Token(tipo, lexem, value, self.line, self.column))
    # utilidades lvl. 1 acabadas
    
    # Utilidades lvl2
    def skip_comment(self):
        if self.prev()=="#":
            char = self.peek()
            while not self.is_at_end():
                if char == "\n":
                    self.line += 1
                    self.column = 1
                    self.advance()
                    break
                self.column+=1
                char = self.advance()
            return True
        return False
    def num_lexer(self):
        consumed = False
        self.current -= 1
        self.column -= 1
        while not self.is_at_end():
            char = self.advance()
            if consumed:
                if not char.isdigit():
                    self.current -= 1
                    break
            else:
                if char==".":
                    consumed = True
                elif char.isdigit():
                    pass
                else:
                    self.current -= 1
                    break
        return consumed
    def string_lex(self):
        start = self.current - 1
        while not self.is_at_end():
            char = self.advance()
            if char == self.comillas:
                break
        return self.source[start:self.current]
    def identifier_lex(self):
        start = self.current-1
        while not self.is_at_end():
            char = self.advance()
            if char not in self.var_set:
                self.current -= 1
                break
        return self.source[start:self.current]
    
    # eso solo fueron utilidades, ahora llega el verdadero escaneo LEXER LVL 3

    def scan(self): # ESCANEO
        while not self.is_at_end():
            self.start = self.current
            self.scan_token()
        self.add_token(tokens.TokenType.EOF) # añadir final de archivo al final
        return self.tokens
    def scan_token(self):
        char = self.advance()
        if self.skip_comment():
            return
        elif char=="\n":
            self.column = 1
            self.line += 1
        elif char in ("\t", " ", "\r"):
            pass
        elif char=="(":
            self.add_token(tokens.TokenType.LPAREN, "(")
        elif char==")":
            self.add_token(tokens.TokenType.RPAREN, ")")
        elif char=="{":
            self.add_token(tokens.TokenType.LBRACE, "{") # ME ABURRO, es muy repetitivo
        elif char=="}":
            self.add_token(tokens.TokenType.RBRACE, "}")
        elif char=="[":
            self.add_token(tokens.TokenType.LBRACKET, "[")
        elif char=="]":
            self.add_token(tokens.TokenType.RBRACKET, "]")
        elif char==";":
            self.add_token(tokens.TokenType.SEMICOLON, ";") # SEMICOLON
        # VALORES
        elif char==self.comillas: # String's
            lexem = self.string_lex()
            value = lexem[1:-1]
            self.add_token(tokens.TokenType.STRING, lexem, value)
        elif char.isdigit(): # numbers
            self.num_lexer()
            lexem = self.source[self.start:self.current]
            value = float(lexem)
            self.add_token(tokens.TokenType.FLOAT, lexem, value)
        
        # SIMBOLOS xD
        elif char=="*":
            self.add_token(tokens.TokenType.STAR, "*")
        elif char=="/":
            self.add_token(tokens.TokenType.SLASH, "/")
        elif char=="-":
            self.add_token(tokens.TokenType.MINUS, "-")
        elif char=="+":
            self.add_token(tokens.TokenType.PLUS, "+")
        elif char==":":
            self.add_token(tokens.TokenType.COLON, ":")
        elif char==".":
            self.add_token(tokens.TokenType.DOT, ".")
        elif char=="^":
            self.add_token(tokens.TokenType.XOR, "^")
        # de doble comprobación
        elif char=="!":
            if self.match("="):
                self.add_token(tokens.TokenType.BANG_EQUAL, "!=")
            else:
                self.add_token(tokens.TokenType.BANG, "!")
        elif char=="=":
            if self.match("="):
                self.add_token(tokens.TokenType.EQUAL_EQUAL, "==")
            else:
                self.add_token(tokens.TokenType.EQUAL, "=")
        elif char=="<":
            if self.match("="):
                self.add_token(tokens.TokenType.LESS_EQUAL, "<=")
            else:
                self.add_token(tokens.TokenType.LESS, "<")
        elif char==">":
            if self.match("="):
                self.add_token(tokens.TokenType.GREATER_EQUAL, ">=")
            else:
                self.add_token(tokens.TokenType.GREATER, ">")
        elif char==",":
        # BUILT-IN
            self.add_token(tokens.TokenType.COMMA, ",")
        elif self.match_sequence("import"):
            self.add_token(tokens.TokenType.IMPORT, "import")
        # Other Types
        elif self.match_sequence("null"):
            self.add_token(tokens.TokenType.NULL, "null")
        elif self.match_sequence("true"):
            self.add_token(tokens.TokenType.BOOL, "true", True)
        elif self.match_sequence("false"):
            self.add_token(tokens.TokenType.BOOL, "false", False)
        
        # Keywords
        elif self.match_sequence("func"):
            self.add_token(tokens.TokenType.FUNC, "func")
        elif self.match_sequence("var"):
            self.add_token(tokens.TokenType.VAR, "var")
        elif self.match_sequence("list"):
            self.add_token(tokens.TokenType.LIST, "list")
        elif self.match_sequence("call"):
            self.add_token(tokens.TokenType.CALL, "call")
        elif self.match_sequence("to"):
            self.add_token(tokens.TokenType.TO, "to")
        elif self.match_sequence("and"):
            self.add_token(tokens.TokenType.AND, "and")
        elif self.match_sequence("not"):
            self.add_token(tokens.TokenType.BANG, "not")
        elif self.match_sequence("or"):
            self.add_token(tokens.TokenType.OR, "or")
        elif self.match_sequence("while"):
            self.add_token(tokens.TokenType.WHILE, "while")
        elif self.match_sequence("for"):
            self.add_token(tokens.TokenType.FOR, "for")
        elif self.match_sequence("return"):
            self.add_token(tokens.TokenType.RETURN, "return")
        elif self.match_sequence("if"):
            self.add_token(tokens.TokenType.IF, "if")
        elif self.match_sequence("else"):
            self.add_token(tokens.TokenType.ELSE, "else")
        elif self.match_sequence("as"):
            self.add_token(tokens.TokenType.AS)
        elif self.match_sequence("break"):
            self.add_token(tokens.TokenType.BREAK)
        elif char in self.var_set:
            self.add_token(tokens.TokenType.IDENTIFIER, self.identifier_lex())
        else:
            self.error = True
            lerror = self.throw("Unexpected Token")
            lerror.print_error()
            if self.debug:
                print(f"[!] {lerror} [!]")
    def throw(self, msg): # Ayuditas del pedro sánchez
        return errors.ZynkError(self.line, self.column, msg)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import templates

class MemoryManager:
    def __init__(self):
        pass
    def emit_nenv(self):
        return 'env = cynkEnvCreate(env, CYNK_ENV_CAP, &sysarena);'
    def emit_ret_env(self):
        return 'env = cynkEnvBack(env, &sysarena);'
    def emit_set(self, name, stack):
        return f'zynkTableSet(&sysarena, env, "{name}", stack.spop());'
    def emit_new(self, name, stack):
        return f'zynkTableNew(env, "{name}", stack.spop(), &sysarena);'
    def emit_get(self, name):
        return f'zynkTableGet(env, "{name}")'
    def emit_pget(self, name, stack):
        return stack.spush(self.emit_get(name))



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

# clase base para definir expresiones

from . import zenv
from .. import errors

class Expr:
    def __init__(self):
        raise NotImplementedError()
    def accept(self, visitor):
        raise NotImplementedError()
    def __str__(self):
        raise NotImplementedError()
    def __repr__(self):
        return self.__str__()
    
# Literal con patron del visitante, WoW

class Literal(Expr):
    def __init__(self, value):
        self.value = value
    def accept(self, visitor):
        return visitor.visit_literal(self)
    def __str__(self):
        return f"[ Literal : {self.value} ]"

# Operador Binario

class Binary(Expr):
    def __init__(self, left, operand, right):
        self.right = right
        self.left = left
        self.operand = operand
    def accept(self, visitor):
        return visitor.visit_binary(self)
    def __str__(self):
        return f"[ Binary : {self.left} : {self.operand} : {self.right} ]"
    
# Operador Unario

class Unary(Expr):
    def __init__(self, operand, right):
        self.right = right
        self.operand = operand
    def accept(self, visitor):
        return visitor.visit_unary(self)
    def __str__(self):
        return f"[ Unary : {self.operand} : {self.right} ]"
    
# Unos bonitos parentesis :)

class Grouping(Expr):
    def __init__(self, expression):
        self.expression = expression
    def accept(self, visitor):
        return visitor.visit_grouping(self)
    def __str__(self):
        return f"[ Grouping : {self.expression} ]"
    
# |--------------------------------------------------------|
# | Fin de las expresiones, ahora son sentencias           |
# | Guillermo Leira Temes 2:51 a.m. 18/4/2025              |
# |--------------------------------------------------------|

class PrintStmt(Expr):
    def __init__(self, expression):
        self.expression = expression
    def accept(self, visitor):
        return visitor.visit_print(self)
    def __str__(self):
        return f"[ Print : {self.expression} ]"
    
class InputStmt(Expr):
    def __init__(self, expression, to):
        self.expression = expression
        self.to = to
    def accept(self, visitor):
        return visitor.visit_input(self)
    def __str__(self):
        return f"[ Input : {self.expression} ]"
    

class VarDef(Expr):
    def __init__(self, name, expression):
        self.name = name
        self.expression = expression
    def accept(self, visitor):
        return visitor.visit_var_definition(self)
    def __str__(self):
        return f"[ Var Definition : {self.name} : {self.expression} ]"
class VarAssign(VarDef):
    def accept(self, visitor):
        return visitor.visit_var_assign(self)
    def __str__(self):
        return f"[ Var Assign : {self.name} : {self.expression} ]"
        
class Identifier(Expr): # lo usare para cargar cosas en memeoria bajo un nombre
    def __init__(self, name):
        self.name = name
    def accept(self, visitor):
        return visitor.visit_identifier(self)
    def __str__(self):
        return f"[ Identifier : {self.name} ]"
    
class FuncDef(Expr):
    def __init__(self, name, params, body):
        self.name = name
        self.params = params
        self.body = body
    def accept(self, visitor):
        return visitor.visit_func_definition(self)
    def __str__(self):
        return f"[ Func Definition : {self.name} : {self.params} : {self.body} ]"
    
class CallFunc(Expr):
    def __init__(self, name, args, to=None):
        self.name = name # donde demonios se guarda
        self.args = args
        self.to = to
    def accept(self, visitor):
        return visitor.visit_call_function(self)
    def __str__(self):
        return f"[ Call : {self.name} : {self.args} : {self.to} ]"
    
class IfExpr(Expr):
    def __init__(self, condition, then, else_branch):
        self.condition = condition
        self.then = then
        self.else_branch = else_branch
    def accept(self, visitor):
        return visitor.visit_if(self)
    def __str__(self):
        return f"[ If : {self.condition} : {self.then} : {self.else_branch} ]"

class ImportExpr(Expr):
    def __init__(self, name, alias):
        self.name = name
        self.alias = alias
    def accept(self, visitor):
        return visitor.visit_import(self)
    def __str__(self):
        return f"[ Import : {self.name} ]"

class WhileExpr(Expr):
    def __init__(self, condition, body):
        self.condition = condition
        self.body = body
    def accept(self, visitor):
        return visitor.visit_while(self)
    def __str__(self):
        return f"[ While : {self.condition} : {self.body} ]"
    
class ForExpr(Expr):
    def __init__(self, inited, inc, condition, body):
        self.initialized = inited
        self.condition = condition
        self.increment = inc
        
        self.body = body
    def accept(self, visitor):
        return visitor.visit_for(self)
    def __str__(self):
        return f"[ For : {self.initialized} : {self.condition} : {self.increment} : {self.body} ]"
    
class BlockExpr(Expr): # casí me olvido de añadir esto, parece insignificante, pero mejora todo muchisimo
    def __init__(self, body):
        self.body = body
    def accept(self, visitor):
        return visitor.visit_block(self)
    def __str__(self):
        return f"[ Block : {self.body} ]"
    
class ZynkFunc:
    def __init__(self, declaration):
        self.declaration = declaration
    def call(self, interpreter, args):
        env = interpreter.subenv()

        for i in range(len(self.declaration.params)):
            env.define(self.declaration.params[i], args[i])
        interpreter.switch(env)
        interpreter.eval(self.declaration.body)
        try:
            return interpreter.ret
        finally:
            interpreter.ret = None
            interpreter.undo()

class CynkFunc(ZynkFunc):
    def call(self, trans, args):
        return f"""
    zynkCallFunction(env, "{self.declaration.name}", {trans.stack.spop()});

    """

class Module:
    def __init__(self, name, env):
        self.name = name
        self.env = env

class MIdentifier(Identifier):
    def __init__(self, module, name):
        self.module = module
        super().__init__(name)
    def accept(self, visitor):
        visitor.switch(visitor.env.get(self.module).env)
        try:
            return super().accept(visitor)
        finally:
            visitor.undo()
    def __str__(self):
        return f"[ Module : {self.module} : [ {self.name} ] ]"

class ReturnExpr(Expr):
    def __init__(self, expression):
        self.expression = expression
    def accept(self, visitor):
        return visitor.visit_return(self)
    def __str__(self):
        return f"[ Return : {self.expression} ]"

class ArrayExpr(Expr): # Ho Ho, ya llegan
    def __init__(self, elements):
        self.items = elements
    def accept(self, visitor):
        return visitor.visit_array_expr(self)
    def __str__(self):
        return f"[ Array : {self.items} ]"
    
class IndexExpr(Expr):
    def __init__(self, array, index):
        self.array = array # un array xD
        self.index = index # posición, espera, porque lo digo. acaso no sabes ingés?
    def accept(self, visitor):
        return visitor.visit_index_expr(self)
    def __str__(self):
        return f"[ Index : {self.index} : {self.array} ]"

class IndexAssignExpr(Expr):
    def __init__(self, array, index, value):
        self.array = array
        self.index = index
        self.value = value
    def accept(self, visitor):
        return visitor.visit_index_assign_expr(self)
    def __str__(self):
        return f"[ Assign Index : {self.value} : {self.index} : {self.array} ]"


class BreakExpr(Expr):
    def accept(self, visitor):
        return visitor.visit_break(self)
    def __str__(self):
        return "[ Break ]"


# FUTURO ¿?
"""
class ArrayDef(Expr):
    pass.....


class ArrayGet(Expr):
    pass......

    
class Struct(Expr):
    pass......

    
class DefStruct(Expr):
    pass.......

    
"""

# Bueno, esta es mi pequeña implementación actual de expresiones para ZynkLite
# Espero que te haya gustado, pues lo escribi a las 3:21 del 18/4/2025
# No se cuando estaras leyendo esto, pero Gracias ;)



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

class Enviroment: # importante, diferente contexto => diferente resultado
    def __init__(self, enclosing=None):
        self.values = {}
        self.enclosing = enclosing
    def define(self, name, value):
        self.values[name]=value
    def get(self, name):
        if name in self.values:
            return self.values[name]
        elif self.enclosing is not None:
            return self.enclosing.get(name)
        return RuntimeError(f"Error '{name}' not defined!")
    def assign(self, name, value):
        if name in self.values:
            self.values[name] = value
        elif self.enclosing is not None:
            self.enclosing.assign(name, value)
        return RuntimeError(f"Error'{name}' not defined!")



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

cynk_headers_defaults = {
            "cynk_header_names":["cynk_stack.h", "cynk_memory.h", "cynk_env.h"],
        }

stack_defaults = {
            "index_type":"uint32_t",
            "stack_max":"256"
        }

stack_headers = """ 
#ifndef CYNK_STACK
#define CYNK_STACK

#include "libzynk/zynk.h"
#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#define STACK_MAX {stack_max}

{index_type} index=0;

Value stack[STACK_MAX];

Value cynkPop();

void cynkPush(Value val);

void cynkSwap();

Value cynkPop() {{
    if (index == 0) return zynkNull();
    index--;
    return stack[index];
}}

void cynkDel() {{
	if (index==STACK_MAX) return;
	zynk_release(stack[index+1], &sysarena);
}}

void cynkPush(Value val) {{
    if (index==STACK_MAX) return;
    if (stack[index]!=val) {{
        zynk_release(stack[index], &sysarena);
    }}
    stack[index++] = val;
}}

void cynkSwap() {{
    if (index<2) return;
    Value __a__=cynkPop();
    Value __b__=cynkPop();
    cynkPush(__a__);
    cynkPush(__b__);
}}

"""

sysarena_defaults = {
            "num_arenas":"2048",
            "memory_size":"1024*1024",
            "a_or_b":"ANSI",
        }
sysarena_header = """

#ifndef CYNK_SYSARENA_SETUP
#define CYNK_SYSARENA_SETUP

#include "libzynk/zynk.h"
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

#define NUM_ARENAS {num_arenas}
#define MEM_SIZE {memory_size}
#define {a_or_b} // STANDALONE or ANSI?

ArenaManager sysarena;

bool cynkSysarenaInit();

#ifdef STANDALONE
Arena arena_pool[NUM_ARENAS];
uint8_t memory_pool[MEM_SIZE];

bool cynkSysarenaInit() {{
    return sysarena_init(&sysarena, memory_pool, arena_pool, MEM_SIZE, NUM_ARENAS);
}}
#endif


#ifdef ANSI
#include <stdlib.h>
#include <stdio.h>
Arena *arena_pool;
uint8_t *memory_pool;

bool cynkSysarenaInit() {{
    arena_pool = (Arena *)malloc(sizeof(Arena)*NUM_ARENAS);
    memory_pool = (uint8_t *)malloc(MEM_SIZE);
    if (arena_pool==NULL || memory_pool==NULL) return false;
    return sysarena_init(&sysarena, memory_pool, arena_pool, MEM_SIZE, NUM_ARENAS);
}}
#endif

#endif

"""

zenv_defaults = {
            "table_cap":"32",
        }

zenv_headers = """

#ifndef CYNK_ENV
#define CYNK_ENV

#include "libzynk/zynk.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#define CYNK_ENV_CAP {table_cap}

// ahora vienen los helpers, porque crear entornos es una pesadilla, xD

ZynkEnv* cynkEnvCreate(ZynkEnv *enclosing, size_t capacity, ArenaManager *manager);
bool cynkFreeEnv(ZynkEnv *env, ArenaManager *manager);
ZynkEnv* cynkEnvBack(ZynkEnv *env, ArenaManager *manager);




// Implementaciones!

ZynkEnv* cynkEnvCreate(ZynkEnv *enclosing, size_t capacity, ArenaManager *manager) {{
    ZynkEnv *new_env = (ZynkEnv *)sysarena_alloc(manager, sizeof(ZynkEnv));
    if (new_env==NULL) return NULL;

    new_env->local = (ZynkEnvTable *)sysarena_alloc(manager, sizeof(ZynkEnvTable));// asignar tabla local
    if (new_env->local == NULL) {{
        sysarena_free(manager, new_env);
        return NULL;
    }}
    new_env->local->entries = (ZynkEnvEntry**)sysarena_alloc(manager, sizeof(ZynkEnvEntry*)*capacity);
    if (new_env->local->entries==NULL) {{
        sysarena_free(manager, new_env->local);
        sysarena_free(manager, new_env);
        return NULL;
    }}
    
    if (!zynkEnvInit(new_env, capacity, enclosing, manager)) {{
        sysarena_free(manager, new_env->local->entries);
        sysarena_free(manager, new_env->local);
        sysarena_free(manager, new_env);
        return NULL;
    }}
    return new_env;
}}

bool cynkFreeEnv(ZynkEnv *env, ArenaManager *manager) {{
    if (env==NULL || manager==NULL) return false;

    bool success=true;
    
    if (env->local!=NULL) {{
        for (size_t i=0;i<env->local->capacity;i++) {{
            if (env->local->entries[i]!=NULL && env->local->entries[i]->name!=NULL) zynk_release(env->local->entries[i]->value, manager);
        if (!freeZynkTable(manager, env->local)) success=false;
        }}
    }}

    if (!sysarena_free(manager, env)) success=false;

    return success;
}}

ZynkEnv* cynkEnvBack(ZynkEnv *env, ArenaManager *manager) {{
    if (env==NULL || env->enclosing==NULL) return NULL;

    ZynkEnv *__tmp__=env;
    env=__tmp__->enclosing;
    if (!cynkFreeEnv(__tmp__, manager)) return NULL;

    return env;
}}

#endif

"""



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

class TokenType:
    #delimitadores
    LPAREN = 0 # (
    RPAREN = 1 # )
    LBRACE = 2 # {
    RBRACE = 3 # }
    LBRACKET = 4 # [
    RBRACKET = 5 # ] 
    COMMA = 6 # ,
    SEMICOLON = 7 # ;
    DOT = 8 # .
    COLON = 9 # :
    EQUAL = 21 # =

    # Tipos de Datos
    FLOAT = 10 # números de punto flotante
    ARRAY = 11 # una lista bro, como los hare???
        # STRUCTS ¿?
    STRING = 12
    NULL = 13
    BOOL = 14

    # Aritmetica/Lógica

    STAR = 15 # *
    SLASH = 16 # /
    MINUS = 17
    PLUS = 18
    GREATER = 19
    LESS = 20
    EQUAL_EQUAL = 22 # ==
    LESS_EQUAL = 23 # <=
    GREATER_EQUAL = 24 # >=
    BANG = 25 # !
    BANG_EQUAL = 26 # !=
    AND = 27
    OR = 28
    XOR = 29

    # Abstracto
    IDENTIFIER = 30

    # Palabras Clave

    VAR = 31
    LIST = 32
    FUNC = 33
    CALL = 34
    TO = 35
    AS = 45
    RETURN = 36
    ELSE = 37
    IF = 38
    WHILE = 39
    FOR = 40
    BREAK = 46

    # built-in
    IMPORT = 43
    
    # IDK
    EOF = 44

class Token:
    def __init__(self, tipo, lexem, value, line, column):
        self.type = tipo
        self.value = value
        self.lexem = lexem
        self.line = line
        self.column = column
    def __repr__(self):
        return self.__str__()
    def __str__(self):
        return f"Token : [ {self.type} : {self.lexem} : {self.value} ] at [{self.line} : {self.column}]"


# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import interpreter as intp
from . import compiler
from . __about__ import __version__ as version
import sys


def main():
    if len(sys.argv) < 1:
        print("[ Error, at least 1 arg is needed ]")
    elif sys.argv[1]=="run":
        filepath = sys.argv[2]
        interpreter = intp.ZynkLInterpreter()
        interpreter.eval_file(filepath)
    # muchas más opciones
    elif sys.argv[1]=="cli":
        print(f"[+] ZynkLite Interpreter {version} [+]")
        print("[*] Type 'quit' or 'exit' to close [*]")
        interpreter = intp.ZynkLInterpreter()
        while True:
            opt = input(">>> ")
            if opt=="quit" or opt=="exit":
                break
            try:
                interpreter.eval(opt)
            except Exception as e:
                print(f"[!] Error: {e} [!]")
        print("[-] ZynkLite Terminated [-]")
    else:
        print("[!] BAD USAGE → zynkl [run/cli] [file] [!]")



# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

from . import expressions as expr
from . import parser
from . import eval
from . import zenv


# SPDX-FileCopyrightText: 2025-present Guille <guilleleiratemes@gmail.com>
#
# SPDX-License-Identifier: GPLv3

class Stack:
    def __init__(self, push_func="cynkPush({value});", pop_func="cynkPop();", swap_func="cynkSwap();"):
        self.push_func=push_func
        self.pop_func=pop_func
        self.swap_func=swap_func
    def spush(self, val):
        return self.push_func.format(value=val)
    def spop(self):
        return self.pop_func[0:-1]
    def clean(self):
        pass
    def swap(self):
        return self.swap_func